/**
    PSPComic - xmlfuncs.cpp
    Copyright (C) 2007  Jeffrey P.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**/

#include "pspcomic.h"

static TiXmlDocument *bookmarks = 0;

char save_config() {
	TiXmlDocument confile;
	TiXmlDeclaration *dec = new TiXmlDeclaration("1.0","","");
	if(!dec) return xml_er_out_of_memory;
	confile.LinkEndChild(dec);
	TiXmlElement *root = new TiXmlElement("config");
	if(!root) return xml_er_out_of_memory;
	root->SetAttribute("version","0.9.9");

	TiXmlElement *item;
	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","resize_method");
	item->SetAttribute("value",*access_int_global(access_resize));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","zoom_persist");
	item->SetAttribute("value",*access_int_global(access_zoom_persist));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","rotate_persist");
	item->SetAttribute("value",*access_int_global(access_rotate_persist));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","pan_rate");
	item->SetAttribute("value",*access_int_global(access_pan_rate));
	root->LinkEndChild(item);

	#ifdef PSP
	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","clock_frequency");
	item->SetAttribute("value",scePowerGetCpuClockFrequency());
	root->LinkEndChild(item);
	#endif

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","scroll_skip");
	item->SetAttribute("value",*access_int_global(access_scroll_skip));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","manga_mode");
	item->SetAttribute("value",*access_int_global(access_manga_mode));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","zoom_box_w");
	item->SetAttribute("value",*access_int_global(access_zoom_box_w));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","zoom_box_h");
	item->SetAttribute("value",*access_int_global(access_zoom_box_h));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","bookmark_on_load");
	item->SetAttribute("value",*access_int_global(access_bookmark_on_load));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","autozoom_mode");
	item->SetAttribute("value",*access_int_global(access_autozoom_mode));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","precaching");
	item->SetAttribute("value",*access_int_global(access_precaching));
	root->LinkEndChild(item);

	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","singlehanded");
	item->SetAttribute("value",*access_int_global(access_singlehanded));
	root->LinkEndChild(item);


	item = new TiXmlElement("item");
	if(!item) return xml_er_out_of_memory;
	item->SetAttribute("id","analog_disabled");
	item->SetAttribute("value",*access_int_global(access_analog_disabled));
	root->LinkEndChild(item);

	confile.LinkEndChild(root);
	confile.SaveFile(config_file);
	return 0;
}
char load_config() {
	TiXmlDocument confile(config_file);
	if(!confile.LoadFile()) {
		confile = TiXmlDocument("config.xml");
		if(!confile.LoadFile()) return xml_er_cannot_load;
	}
	for(
	  TiXmlNode *root = confile.FirstChild("config");
	  root != 0;
	  root = root->NextSibling("config")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		for(
		  TiXmlNode *item = root->FirstChild("item");
		  item != 0;
		  item = item->NextSibling("item")
		) {
			if(item->Type() != TiXmlNode::ELEMENT) continue;
			TiXmlElement *el = item->ToElement();
			const char *id = el->Attribute("id");
			if(!id) return xml_er_malformed;
			else if(!strcmp(id,"resize_method")) {
				int val;
				if(
				  el->Attribute("value",&val) && 
				  (val >= resize_nn) && (val <= resample)
				)
					*access_int_global(access_resize) = val;
			} else if(!strcmp(id,"zoom_persist"))
				el->Attribute("value",access_int_global(access_zoom_persist));
			else if(!strcmp(id,"rotate_persist"))
				el->Attribute("value",access_int_global(access_rotate_persist));
			else if(!strcmp(id,"pan_rate")) {
				int val;
				if(
				  el->Attribute("value",&val) &&
				  (val >= pan_min) && (val <= pan_max)
				)
					*access_int_global(access_pan_rate) = val;
			} else if(!strcmp(id,"clock_frequency")) {
				int val;
				if(el->Attribute("value",&val)) set_clock(val);
			} else if(!strcmp(id,"scroll_skip")) {
				int val;
				if(
				  el->Attribute("value",&val) &&
				  (val > 0) && (val <= options_per_page)
				)
					*access_int_global(access_scroll_skip) = val;
			} else if(!strcmp(id,"manga_mode"))
				el->Attribute("value",access_int_global(access_manga_mode));
			else if(!strcmp(id,"bookmark_on_load"))
				el->Attribute("value",access_int_global(access_bookmark_on_load));
			else if(!strcmp(id,"fullview_mode")) {
				int is_fullview;
				el->Attribute("value",&is_fullview);
				*access_int_global(access_autozoom_mode) = (is_fullview?full_view:full_width);
			} else if(!strcmp(id,"autozoom_mode")) {
				int val;
				if(
				  el->Attribute("value",&val) &&
				  (val > 0) && (val <= autodetect_zoom)
				)
					*access_int_global(access_autozoom_mode) = val;
			} else if(!strcmp(id,"precaching"))
				el->Attribute("value",access_int_global(access_precaching));
			else if(!strcmp(id,"singlehanded"))
				el->Attribute("value",access_int_global(access_singlehanded));
			else if(!strcmp(id,"analog_disabled"))
				el->Attribute("value",access_int_global(access_analog_disabled));
			else if(!strcmp(id,"zoom_box_w")) {
				int val;
				if(
				  el->Attribute("value",&val) &&
				  (val >= 20) && (val <= 180)
				)
					*access_int_global(access_zoom_box_w) = val;
			} else if(!strcmp(id,"zoom_box_h")) {
				int val;
				if(
				  el->Attribute("value",&val) &&
				  (val >= 20) && (val <= 250)
				)
					*access_int_global(access_zoom_box_h) = val;
			}
		}
		return 0;
	}
	return xml_er_malformed;
}

char load_bookmarks() {
	bookmarks = new TiXmlDocument(bookmark_file);
	if(!bookmarks) return xml_er_out_of_memory;
	if(!bookmarks->LoadFile()) {
		delete bookmarks;
		bookmarks = new TiXmlDocument("bookmarks.xml");
		if(!bookmarks->LoadFile()) {
			delete bookmarks;
			bookmarks = 0;
			return xml_er_cannot_load;
		}
	}
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) if(root->Type() == TiXmlNode::ELEMENT) return 0;
	delete bookmarks;
	bookmarks = NULL;
	return xml_er_malformed;
}
char save_bookmark(comic_book *book) {
	if(!book) return xml_er_unknown;
	delete_bookmarks(book->filename);
	if(!bookmarks) {
		bookmarks = new TiXmlDocument();
		if(!bookmarks) return xml_er_out_of_memory;
		TiXmlDeclaration *dec = new TiXmlDeclaration("1.0","","");
		if(!dec) return xml_er_out_of_memory;
		bookmarks->LinkEndChild(dec);
		TiXmlElement *root = new TiXmlElement("bookmarks");
		if(!root) return xml_er_out_of_memory;
		root->SetAttribute("version","0.9.9");
		bookmarks->LinkEndChild(root);
		TiXmlElement *bookmark = new TiXmlElement("bookmark");
		if(!bookmark) return xml_er_out_of_memory;
		bookmark->SetAttribute("book",book->filename);
		bookmark->SetAttribute("page",book->current_page);
		root->LinkEndChild(bookmark);
		bookmarks->SaveFile(bookmark_file);
		return 0;
	} else {
		for(
		  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
		  root != 0;
		  root = root->NextSibling("bookmarks")
		) {
			if(root->Type() != TiXmlNode::ELEMENT) continue;
			root->ToElement()->SetAttribute("version","0.9.9");
			TiXmlElement *bookmark = new TiXmlElement("bookmark");
			if(!bookmark) return xml_er_out_of_memory;
			bookmark->SetAttribute("book",book->filename);
			bookmark->SetAttribute("page",book->current_page);
			root->LinkEndChild(bookmark);
			bookmarks->SaveFile(bookmark_file);
			return 0;
		}
		return xml_er_malformed;
	}
}
char delete_bookmarks(char *book) {
	load_bookmarks();
	if(!bookmarks) return 0;
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		for(
		  TiXmlNode *item = root->FirstChild("bookmark");
		  item != 0;
		  item = item->NextSibling("bookmark")
		) {
			if(item->Type() != TiXmlNode::ELEMENT) continue;
			TiXmlElement *el = item->ToElement();
			if(!el) return xml_er_unknown;
			if(el->Attribute("book")) {
				if(!strcmp(el->Attribute("book"),book) && !el->Attribute("id"))
					root->RemoveChild(item);
			} else return xml_er_malformed;
		}
		bookmarks->SaveFile(bookmark_file);
		return 0;
	}
	return xml_er_malformed;
}
Sint32 get_bookmark(char *book) {
	char status = load_bookmarks();
	if(status < 0) return status;
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		for(
		  TiXmlNode *item = root->FirstChild("bookmark");
		  item != 0;
		  item = item->NextSibling("bookmark")
		) {
			if(item->Type() != TiXmlNode::ELEMENT) continue;
			TiXmlElement *el = item->ToElement();
			if(!el) return xml_er_unknown;
			if(el->Attribute("book")) {
				if(!strcmp(el->Attribute("book"),book)) {
					int val = xml_er_malformed;
					if(!el->Attribute("page",&val)) continue;
					if(val < 0) return xml_er_malformed;
					return val;
				}
			} else return xml_er_malformed;
		}
		return xml_er_unknown;
	}
	return xml_er_malformed;
}
char delete_all_bookmarks() {
	char status = load_bookmarks();
	if(status < 0) return status;
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		root->Clear();
		bookmarks->SaveFile(bookmark_file);
		return 0;
	}
	return xml_er_malformed;
}
char purge_bookmarks() {
	char status = load_bookmarks();
	if(status < 0) return status;
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		for(
		  TiXmlNode *item = root->FirstChild("bookmark");
		  item != 0;
		  item = item->NextSibling("bookmark")
		) {
			if(item->Type() != TiXmlNode::ELEMENT) continue;
			TiXmlElement *el = item->ToElement();
			if(!el) return xml_er_unknown;
			if(el->Attribute("book")) {
				FILE* temp = fopen(el->Attribute("book"),"rb");
				if(temp) fclose(temp);
				else root->RemoveChild(item);
			} else return xml_er_malformed;
		}
		bookmarks->SaveFile(bookmark_file);
		return 0;
	}
	return xml_er_malformed;
}
char get_book_by_id(char *id, char *book, size_t *book_len) {
 	char status = load_bookmarks();
	if(status < 0) return status;
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		for(
		  TiXmlNode *item = root->FirstChild("bookmark");
		  item != 0;
		  item = item->NextSibling("bookmark")
		) {
			if(item->Type() != TiXmlNode::ELEMENT) continue;
			TiXmlElement *el = item->ToElement();
			if(!el) return xml_er_unknown;
			if(el->Attribute("id")) {
				if(!strcmp(el->Attribute("id"),id)) {
					const char *bookTemp = el->Attribute("book");
					if(bookTemp) {
						memset(book,0,*book_len);
						strncpy(book,bookTemp,*book_len);
						*book_len = strlen(book);
						return 0;
					}
					return xml_er_malformed;
				}
			}
		}
		return xml_er_unknown;
	}
	return xml_er_malformed;
}
char get_saved_book(char *book, size_t *book_len) {
	return get_book_by_id("saved_book", book, book_len);
}
char get_last_book(char *book, size_t *book_len) {
	return get_book_by_id("last_book", book, book_len);
}

char set_book_by_id(char *id, char *book) {
 	load_bookmarks();
	if(!bookmarks) {
		bookmarks = new TiXmlDocument();
		if(!bookmarks) return xml_er_out_of_memory;
		TiXmlDeclaration *dec = new TiXmlDeclaration("1.0","","");
		if(!dec) return xml_er_out_of_memory;
		bookmarks->LinkEndChild(dec);
		TiXmlElement *root = new TiXmlElement("bookmarks");
		if(!root) return xml_er_out_of_memory;
		root->SetAttribute("version","0.9.9");
		bookmarks->LinkEndChild(root);
	}
	for(
	  TiXmlNode *root = bookmarks->FirstChild("bookmarks");
	  root != 0;
	  root = root->NextSibling("bookmarks")
	) {
		if(root->Type() != TiXmlNode::ELEMENT) continue;
		for(
		  TiXmlNode *item = root->FirstChild("bookmark");
		  item != 0;
		  item = item->NextSibling("bookmark")
		) {
			if(item->Type() != TiXmlNode::ELEMENT) continue;
			TiXmlElement *el = item->ToElement();
			if(!el) return xml_er_unknown;
			if(el->Attribute("id")) {
				if(!strcmp(el->Attribute("id"),id)) {
					el->SetAttribute("book",book);
					bookmarks->SaveFile(bookmark_file);
					return 0;
				}
			}
		}
		TiXmlElement *el = new TiXmlElement("bookmark");
		if(!el) return xml_er_out_of_memory;
		el->SetAttribute("id",id);
		el->SetAttribute("book",book);
		root->LinkEndChild(el);
		bookmarks->SaveFile(bookmark_file);
		return 0;
	}
	return xml_er_malformed;
}
char set_saved_book(char *book) {
	return set_book_by_id("saved_book", book);
}
char set_last_book(char *book) {
	return set_book_by_id("last_book",book);
}
