/**
    PSPComic - xmlfuncs.hpp
    Copyright (C) 2007  Jeffrey P.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**/

#ifndef PSPComic_xmlfuncs_hpp
#define PSPComic_xmlfuncs_hpp

#include "pspcomic.h"

#ifdef __cplusplus
extern "C" {
#endif

enum {
	xml_er_unknown = -1,
	xml_er_cannot_load = -2,
	xml_er_malformed = -4,
	xml_er_out_of_memory = -8
};

//Configuration file functions
char load_config();
char save_config();

//Bookmark functions
char load_bookmarks();
char save_bookmark(comic_book *book);
Sint32 get_bookmark(char *book);

/**
 * delete_bookmarks
 * Deletes the bookmarks for a given comic book
 *
 * char *book: The filename of the comic book
 * Returns 0 on success and an xml_er on failure
 */
char delete_bookmarks(char *book);

/**
 * get_book_by_id
 * Puts the name of the comic book refered to by the identifier id into the
 * buffer pointed to by book
 *
 * char *id: The identifier of the comic book
 * char	*book: The buffer to put the name of the comic book into
 * Uint8 *book_len: A pointer to the length of the buffer. It is set to the
 *		length of the file name after being called
 * Returns 0 on success and an xml_er on failure
 */
char get_book_by_id(char *id, char *book, size_t *book_len);

/**
 * get_saved_book
 * Puts the name of the saved comic book into the buffer pointed to by book.
 * This is a wrapper around get_book_by_id
 *
 * char	*book: The buffer to put the name of the comic book into
 * Uint8 *book_len: A pointer to the length of the buffer. It is set to the
 *		length of the file name after being called
 * Returns 0 on success and an xml_er on failure
 */
char get_saved_book(char *book, size_t *book_len);

/**
 * get_last_book
 * Puts the name of the last opened comic book into the buffer pointed to by
 * book. This is a wrapper around get_book_by_id
 *
 * char	*book: The buffer to put the name of the comic book into
 * Uint8 *book_len: A pointer to the length of the buffer. It is set to the
 *		length of the file name after being called
 * Returns 0 on success and an xml_er on failure
 */
char get_last_book(char *book, size_t *book_len);

char set_book_by_id(char *id, char *book);
char set_saved_book(char *book);
char set_last_book(char *book);

/**
 * purge_bookmarks
 * Purges the bookmark file of bookmarks which point to comic books which are no
 * longer on the filesystem.
 *
 * Returns 0 on success and an xml_er on failure
 */
char purge_bookmarks();
char delete_all_bookmarks();

#ifdef __cplusplus
}
#endif

#endif
